using System.Collections.Generic;
using Knoks.Core.Entities;
using System.Threading.Tasks;

namespace Knoks.Core.Data.Interfaces
{
    public interface IGeneralDao
    {
        Task<IEnumerable<Country>> GetCountries(string countryCode2 = null);
        Task<IEnumerable<Language>> GetLanguages();
        Task<IEnumerable<KnokStatus>> GetKnokStatuses();

        Task<ExchangeStart> GetKnokStartValues(long exchangeId, string currency1, string currency2);
        Task<IEnumerable<RankCount>> GetUserRank(long? userId);
        Task<IEnumerable<KnokLossCount>> GetKnoksLossCount(long? userId); 
        Task AddKnokFeedBack(Feedback feedback);
      
    }
}
